@@ -6,6 +6,7 @@
 import sys
 import asyncio
 import json
+import time
 import aiohttp
 from aioflureedb.signing import DbSigner
 
@@ -78,6 +79,19 @@ class FlureeTransactionFailure(FlureeException):
         FlureeException.__init__(self, message)
 
 
+class FlureeUnexpectedPredicateNumber(FlureeException):
+    """Fluree transaction failed"""
+    def __init__(self, message):
+        """Constructor
+
+        Parameters
+        ----------
+        message : str
+               Error message
+        """
+        FlureeException.__init__(self, message)
+
+
 def _dryrun(method, url, headers, body):
     """Helper function for debugging the library
 
@@ -787,6 +801,9 @@ class _FlureeDbClient:
         self.monitor["rewind"] = 0
         self.monitor["on_block_processed"] = None
         self.monitor["predicate_map"] = dict()
+        self.monitor["predicate_map_block"] = 0
+        self.monitor["lastblock_instant"] = None
+        self.monitor["instant_monitors"] = list()
         if https and not ssl_verify:
             self.ssl_verify_disabled = True
         self.signer = None
@@ -819,7 +836,7 @@ class _FlureeDbClient:
         self.pw_endpoints = set(["generate", "renew", "login"])
         self.implemented = set(["query", "flureeql", "block", "command", "ledger_stats", "list_snapshots", "snapshot"])
 
-    def monitor_init(self, on_block_processed, start_block=None, rewind=0):
+    def monitor_init(self, on_block_processed, start_block=None, rewind=0, always_query_object=False, start_instant=None):
         """Set the basic variables for a fluree block event monitor run
 
         Parameters
@@ -833,6 +850,15 @@ class _FlureeDbClient:
         rewind: int
                 Number of seconds to rewind from now. Currently not implemented.
 
+        always_query_object: bool
+                Boolean choosing if we want to run efficiently and only query if block parsing gives ambiguous results,
+                or if we always want to use extra queries
+
+        start_instant: int
+                If (and only if) instant monitor callbacks are used, this parameter should be provided to avoid
+                large replays of inactive chain instant events that occured after the last block. Use the instant
+                as provided py the persistence callback in the previous run.
+
         Raises
         ------
         NotImplementedError
@@ -843,11 +869,11 @@ class _FlureeDbClient:
         assert callable(on_block_processed)
         assert start_block is None or isinstance(start_block, int)
         assert isinstance(rewind, int)
-        if rewind != 0:
-            raise NotImplementedError("rewind is not yet implemented")
         self.monitor["next"] = start_block
         self.monitor["rewind"] = rewind
+        self.monitor["always_query_object"] = always_query_object
         self.monitor["on_block_processed"] = on_block_processed
+        self.monitor["lastblock_instant"] = start_instant
 
     def monitor_register_create(self, collection, callback):
         """Add a callback for create events on a collection
@@ -865,7 +891,8 @@ class _FlureeDbClient:
         assert callable(callback)
         if collection not in self.monitor["listeners"]:
             self.monitor["listeners"][collection] = dict()
-        self.monitor["listeners"][collection]["C"] = set()
+        if "C" not in self.monitor["listeners"][collection]:
+            self.monitor["listeners"][collection]["C"] = set()
         self.monitor["listeners"][collection]["C"].add(callback)
 
     def monitor_register_delete(self, collection, callback):
@@ -884,10 +911,11 @@ class _FlureeDbClient:
         assert callable(callback)
         if collection not in self.monitor["listeners"]:
             self.monitor["listeners"][collection] = dict()
-        self.monitor["listeners"][collection]["D"] = set()
+        if "D" not in self.monitor["listeners"][collection]:
+            self.monitor["listeners"][collection]["D"] = set()
         self.monitor["listeners"][collection]["D"].add(callback)
 
-    def monitor_register_update(self, collection, callback, predicates=None):
+    def monitor_register_update(self, collection, callback):
         """Add a callback for update events on a collection
 
         Parameters
@@ -898,35 +926,463 @@ class _FlureeDbClient:
         callback: callable
                 Callback to invoke when update event on collection is identified.
 
-        predicates: list
-                List of predicates. If defined, at laest one of these predicates should have been set or updated.
-
-        Raises
-        ------
-        NotImplementedError
-            Currently raised when predicates is specified.
-
         """
 
         assert isinstance(collection, str)
         assert callable(callback)
-        assert predicates is None or isinstance(predicates, list)
-        if isinstance(predicates, list):
-            assert bool(predicates)
-            for predicate in predicates:
-                assert isinstance(predicate, str)
-            raise NotImplementedError("predicates not yet implemented for monitor_register_update")
         if collection not in self.monitor["listeners"]:
             self.monitor["listeners"][collection] = dict()
-        self.monitor["listeners"][collection]["U"] = dict()
-        self.monitor["listeners"][collection]["U"]["callback"] = set()
-        self.monitor["listeners"][collection]["U"]["callback"].add(callback)
-        self.monitor["listeners"][collection]["U"]["predicates"] = predicates
+        if "U" not in self.monitor["listeners"][collection]:
+            self.monitor["listeners"][collection]["U"] = set()
+        self.monitor["listeners"][collection]["U"].add(callback)
+
+    def monitor_instant(self, predicate, callback, offset=0):
+        """Ass a callback for the passing of time on an instant predicate
+
+        Parameters
+        ----------
+        predicate: str
+                Name of the instant predicate to monitor
+        callback: callable
+                Callback to invoke when the time (plus offset) passes the monitored instant
+        offset: int
+                If specified, number of seconds from monitored instant value to trigger on
+        """
+        self.monitor["instant_monitors"].append([predicate, offset*1000, callback])
 
     def monitor_close(self):
         """Abort running any running monitor"""
         self.monitor["running"] = False
 
+    async def _figure_out_next_block(self):
+        """Figure out what block the user wants/needs to be the next block"""
+        if self.monitor["rewind"] != 0 and self.monitor["rewind"] is not None:
+            filt = "(> ?instant (- (now) (* 1000 " + str(self.monitor["rewind"]) + "))))"
+            rewind_block = await self.flureeql.query(
+                select=["?blockid"],
+                opts={"orderBy": ["ASC", "?instant"], "limit": 1},
+                where=[
+                    ["?block", "_block/instant", "?instant"],
+                    ["?block", "_block/number", "?blockid"],
+                    {"filter": [filt]}
+                ])
+            if rewind_block and (self.monitor["next"] is None or self.monitor["next"] < rewind_block[0][0]):
+                self.monitor["next"] = rewind_block[0][0]
+            if not rewind_block:
+                self.monitor["next"] = None
+
+    async def _build_predicates_map(self, block=None):
+        """Build a predicates map for quick lookup
+
+        Returns
+        -------
+        dict
+            dictionary mapping predicate id's to predicate names
+        """
+        if block is not None:
+            if self.monitor["predicate_map_block"] != block:
+                predicates = await self.flureeql.query(select=["id", "name"], ffrom="_predicate", block=block)
+                self.monitor["predicate_map_block"] = block
+            else:
+                predicates = None
+        else:
+            predicates = await self.flureeql.query(select=["id", "name"], ffrom="_predicate")
+        if predicates is not None:
+            predicate = dict()
+            for pred in predicates:
+                predicate[pred["_id"]] = pred["name"]
+            self.monitor["predicate_map"] = predicate
+
+    async def _find_start_block(self):
+        """Find the start block
+
+        Returns
+        -------
+        int
+            Number of the starting block
+
+        Raises
+        ------
+        RuntimeError
+             Raised when the very first ledger_stats issued to FlureeDB returns an error.
+        """
+        if self.monitor["next"] is None:
+            stats = await self.ledger_stats()
+            if "status" in stats and stats["status"] == 200 and "data" in stats and "block" in stats["data"]:
+                startblock = stats["data"]["block"]
+            else:
+                raise RuntimeError("Invalid initial response from ledger_stats")
+        else:
+            startblock = self.monitor["next"]
+        return startblock
+
+    async def _get_endblock(self, errorcount=0):
+        """Get what for now should be the ending block
+
+        Parameters
+        ----------
+        errorcount:  int
+                     Counter for counting succesive API failure
+        Returns
+        -------
+        int
+            The ending block number
+        int
+            An updated version of the errorcount argument
+        """
+        stats = await self.ledger_stats()
+        if "status" in stats and stats["status"] == 200 and "data" in stats and "block" in stats["data"]:
+            endblock = stats["data"]["block"]
+            return endblock, 0
+        return 0, errorcount + 1
+
+    def _get_flakeset_collection(self, flakelist):
+        """Helper function for getting the collection name from a flakes array
+
+        Parameters
+        ----------
+        flakelist :  list
+              list of flake lists
+
+        Returns
+        -------
+        str
+            Name of the collection.
+        """
+        return flakelist[0][1].split("/")[0]
+
+    async def _group_block_flakes(self, block_data, blockno):
+        """Return a grouped-by object-id and predicate name patched version of a block
+
+        Parameters
+        ----------
+        block_data :  list
+              Raw block data as returned by FlureeDB
+        predicate :  dict
+              Dictionary for looking up predicate names by number
+        blockno : int
+              Number of the block currently being processed.
+
+        Returns
+        -------
+        dict
+            A dictionary of object id's to flake arrays.
+
+        Raises
+        ------
+        FlureeUnexpectedPredicateNumber
+            Raised when an unknown predicate id is detected.
+        """
+        has_predicate_updates = False
+        grouped = dict()
+        for flake in block_data[0]["flakes"]:
+            predno = flake[1]
+            # Patch numeric predicates to textual ones.
+            if predno in self.monitor["predicate_map"]:
+                flake[1] = self.monitor["predicate_map"][predno]
+            else:
+                raise FlureeUnexpectedPredicateNumber("Need a restart after new predicates are added to the database")
+            # Group the flakes together by object.
+            if not flake[0] in grouped:
+                grouped[flake[0]] = list()
+            grouped[flake[0]].append(flake)
+        for obj in grouped:
+            if grouped[obj][0][1].split("/")[0] == "_predicate":
+                has_predicate_updates = True
+        if has_predicate_updates:
+            await self._build_predicates_map(blockno)
+        return grouped
+
+    def _get_transactions_and_temp_ids(self, flakeset):
+        """Extract transactions and temp id's from a single 'tx' flakeset
+
+        Parameters
+        ----------
+        flakeset :  list
+              List of flakes belonging to a 'tx' in the current block.
+
+        Returns
+        -------
+        list
+            list of operations from this transaction
+        dict
+            map of temporary ids.
+        """
+        operations = None
+        tempids = None
+        for flake in flakeset:
+            if flake[1] == '_tx/tempids':
+                try:
+                    tid_obj = json.loads(flake[2])
+                    if isinstance(tid_obj, dict):
+                        tempids = tid_obj
+                except json.decoder.JSONDecodeError:
+                    pass
+            elif flake[1] == "_tx/tx":
+                try:
+                    tx_obj = json.loads(flake[2])
+                    if isinstance(tx_obj, dict) and "tx" in tx_obj and isinstance(tx_obj["tx"], list):
+                        operations = tx_obj["tx"]
+                except json.decoder.JSONDecodeError:
+                    pass
+        return operations, tempids
+
+    def _get_block_instant(self, flakeset):
+        """Extract transactions and temp id's from a single 'tx' flakeset
+
+        Parameters
+        ----------
+        flakeset :  list
+              List of flakes belonging to a 'tx' in the current block.
+
+        Returns
+        -------
+        int
+            Time instance value for this block
+        """
+        instance = None
+        for flake in flakeset:
+            if flake[1] == '_block/instant':
+                instance = flake[2]
+        return instance
+
+    async def _get_block_instant_by_blockno(self, block):
+        """Get the instant timestamp for a given block number
+
+        Parameters
+        ----------
+        block: int
+            Block number
+
+        Returns
+        -------
+        int
+            Time instance value for this block
+        """
+        result = await self.flureeql.query(
+            select=["?instant"],
+            where=[
+                ["?block", "_block/instant", "?instant"],
+                ["?block", "_block/number", block]
+            ]
+        )
+        if result:
+            return result[0][0]
+        return None
+
+    def _get_object_id_to_operation_map(self, tempids, operations):
+        """Process temp ids and operations, return an object id to operation map.
+
+        Parameters
+        ----------
+        tempids :  dict
+                  Temp-id map
+        operations :     list
+                  The url that would have been used
+
+        Returns
+        -------
+        dict
+            object id to operation map.
+        """
+        # pylint: disable=too-many-nested-blocks, too-many-branches
+        obj_tx = dict()
+        if tempids:
+            for tmp_id in tempids:
+                real_id = tempids[tmp_id]
+                counters = dict()
+                for operation in operations:
+                    if isinstance(operation, dict) and "_id" in operation:
+                        if isinstance(operation["_id"], str):
+                            if operation["_id"] == tmp_id:
+                                obj_tx[real_id] = operation
+                            if operation["_id"] not in counters:
+                                counters[operation["_id"]] = 0
+                            counters[operation["_id"]] += 1
+                            altname = operation["_id"] + "$" + str(counters[operation["_id"]])
+                            if altname == tmp_id:
+                                obj_tx[real_id] = operation
+                        elif isinstance(operation["_id"], list):
+                            if len(operation["_id"]) == 2:
+                                txid = '["' + operation["_id"][0] + '" "' + operation["_id"][1] + '"]'
+                                if txid == tmp_id:
+                                    obj_tx[real_id] = operation
+        if len(operations) == 1:
+            obj_tx[""] = operations[0]
+        for operation in operations:
+            if isinstance(operation, dict) and "_id" in operation:
+                if isinstance(operation["_id"], int):
+                    obj_tx[operation["_id"]] = operation
+        return obj_tx
+
+    async def _do_instant_monitor(self, oldinstant, newinstant, blockno):
+        for monitor in self.monitor["instant_monitors"]:
+            predicate = monitor[0]
+            offset = monitor[1]
+            callback = monitor[2]
+            windowstart = oldinstant - offset
+            windowstop = newinstant - offset
+            filt = "(and (> ?instant " + str(windowstart) + ") (<= ?instant " + str(windowstop) + "))"
+            eventlist = await self.flureeql.query(
+                select=[{"?whatever": ["*"]}],
+                opts={"orderBy": ["ASC", "?instant"]},
+                where=[
+                    ["?whatever", predicate, "?instant"],
+                    {"filter": [filt]}
+                ],
+                block=blockno
+                )
+            for event in eventlist:
+                await callback(event)
+
+    async def _process_instance(self, instant, block, fromblock):
+        minute = 60000
+        timeout = 1*minute
+        if (fromblock or
+                self.monitor["lastblock_instant"] and
+                self.monitor["lastblock_instant"] + timeout < instant):
+            if self.monitor["lastblock_instant"]:
+                await self._do_instant_monitor(self.monitor["lastblock_instant"], instant, block)
+            self.monitor["lastblock_instant"] = instant
+
+    async def _get_and_preprocess_block(self, blockno):
+        """Fetch a block by block number and preprocess it
+
+        Parameters
+        ----------
+        blockno :  int
+                  Number of the block that needs to be fetched
+        predicate : dict
+                  Predicate id to name map
+
+        Returns
+        -------
+        list
+            A grouped and predicate patched version of the fetched block.
+        dict
+            Object id to operation dict
+        """
+        # Fetch the new block
+        block_data = await self.block.query(block=blockno)
+        # Groub by object
+        try:
+            grouped = await self._group_block_flakes(block_data, blockno)
+        except FlureeUnexpectedPredicateNumber:
+            await self._build_predicates_map(blockno)
+            grouped = await self._group_block_flakes(block_data, blockno)
+        # Distill new ones using _tx/tempids
+        obj_tx = dict()
+        block_meta = dict()
+        for obj in grouped:
+            transactions = None
+            tempids = None
+            instant = None
+            collection = self._get_flakeset_collection(grouped[obj])
+            if collection == "_tx":
+                transactions, tempids = self._get_transactions_and_temp_ids(grouped[obj])
+            if collection == "_block":
+                instant = self._get_block_instant(grouped[obj])
+                for flake in grouped[obj]:
+                    if len(flake[1].split("/")) > 1:
+                        block_meta[flake[1].split("/")[1]] = flake[2]
+            if transactions:
+                obj_tx = self._get_object_id_to_operation_map(tempids, transactions)
+            if instant:
+                await self._process_instance(instant, blockno, True)
+        return grouped, obj_tx, instant, block_meta
+
+    async def _process_flakeset(self, collection, obj, obj_tx, blockno, block_meta):
+        """Process temp ids and operations, return an object id to operation map.
+
+        Parameters
+        ----------
+        collection :  str
+                  name of the collection the object for this flakeset refers to
+        obj :     list
+                  The flakelist
+        obj_tx :  dict
+                  Dictionary mapping from object id to operation object.
+        blockno : int
+                  Block number of the block currently being processed.
+
+        """
+        # pylint: disable=too-many-branches,too-many-statements
+        operation = None
+        action = None
+        previous = None
+        latest = None
+        if obj[0][0] in obj_tx:
+            operation = obj_tx[obj[0][0]]
+        elif "" in obj_tx:
+            operation = obj_tx[""]
+        has_true = False
+        has_false = False
+        for flake in obj:
+            if flake[4]:
+                has_true = True
+            else:
+                has_false = True
+        if self.monitor["always_query_object"]:
+            previous = await self.flureeql.query(select=["*"], ffrom=obj[0][0], block=blockno-1)
+            if previous:
+                previous = previous[0]
+                action = "update"
+            else:
+                previous = None
+            latest = await self.flureeql.query(select=["*"], ffrom=obj[0][0], block=blockno)
+            if latest:
+                latest = latest[0]
+            else:
+                latest = None
+            if previous is None:
+                action = "insert"
+            elif latest is None:
+                action = "delete"
+            else:
+                action = "update"
+        if operation and "_action" in operation and operation["_action"] != "upsert":
+            action = operation["_action"]
+        if action is None and has_true and has_false:
+            action = "update"
+        if action is None and operation and "_id" in operation and isinstance(operation["_id"], str):
+            action = "insert"
+        if action is None and operation and has_false and not has_true:
+            action = "delete"
+        if action is None and has_true and not has_false:
+            if blockno > 1:
+                previous = await self.flureeql.query(select=["*"], ffrom=obj[0][0], block=blockno-1)
+                if previous:
+                    previous = previous[0]
+                    action = "update"
+                else:
+                    previous = None
+                    action = "insert"
+            else:
+                previous = None
+                action = "insert"
+        if action is None:
+            latest = await self.flureeql.query(select=["*"], ffrom=obj[0][0], block=blockno)
+            if latest:
+                latest = latest[0]
+                action = "update"
+            else:
+                latest = None
+                action = "delete"
+        if action == "insert" and "C" in self.monitor["listeners"][collection]:
+            for callback in self.monitor["listeners"][collection]["C"]:
+                await callback(obj_id=obj[0][0], flakes=obj, new_obj=latest, operation=operation, block_meta=block_meta)
+        elif action == "update" and "U" in self.monitor["listeners"][collection]:
+            for callback in self.monitor["listeners"][collection]["U"]:
+                await callback(obj_id=obj[0][0],
+                               flakes=obj,
+                               old_obj=previous,
+                               new_obj=latest,
+                               operation=operation,
+                               block_meta=block_meta)
+        elif action == "delete" and "D" in self.monitor["listeners"][collection]:
+            for callback in self.monitor["listeners"][collection]["D"]:
+                await callback(obj_id=obj[0][0], flakes=obj, old_obj=previous, operation=operation, block_meta=block_meta)
+
     async def monitor_untill_stopped(self):
         """Run the block event monitor untill stopped
 
@@ -939,108 +1395,64 @@ class _FlureeDbClient:
             Raised either when there are no listeners set, or if there are too many errors.
 
         """
-        # TODO: fix these when we look at getting rid of the extra queries.
-        # pylint: disable = too-many-nested-blocks, too-many-locals, too-many-return-statements, too-many-branches, too-many-statements
-        # Some basic asserts
-        if not bool(self.monitor["listeners"]):
+        # pylint: disable=too-many-nested-blocks, too-many-branches, too-many-return-statements
+        if (not bool(self.monitor["listeners"])) and (not bool(self.monitor["instant_monitors"])):
             raise RuntimeError("Can't start monitor with zero registered listeners")
-        if self.monitor["rewind"] != 0:
-            raise NotImplementedError("rewind is not implemented yet!")
         # Set running to true. We shall abort when it is set to false.
         self.monitor["running"] = True
+        await self._figure_out_next_block()
+        if not self.monitor["running"]:
+            return
+        startblock = await self._find_start_block()
+        if not self.monitor["running"]:
+            return
         # First make a dict from the _predicate collection.
-        predicates = await self.flureeql.query(select=["id", "name"], ffrom="_predicate")
+        if startblock > 1:
+            await self._build_predicates_map(startblock - 1)
         if not self.monitor["running"]:
             return
-        predicate = dict()
-        for pred in predicates:
-            predicate[pred["_id"]] = pred["name"]
         noblocks = True
-        if self.monitor["next"] is None:
-            stats = await self.ledger_stats()
-            if not self.monitor["running"]:
-                return
-            if "status" in stats and stats["status"] == 200 and "data" in stats and "block" in stats["data"]:
-                startblock = stats["data"]["block"]
-            else:
-                raise RuntimeError("Invalid initial response from ledger_stats")
-        else:
-            startblock = self.monitor["next"]
+        if startblock > 1 and self.monitor["instant_monitors"] and self.monitor["lastblock_instant"] is None:
+            self.monitor["lastblock_instant"] = await self._get_block_instant_by_blockno(startblock-1)
+        if not self.monitor["running"]:
+            return
         stats_error_count = 0
+        last_instant = 0
         while self.monitor["running"]:
-            # If we had zero blocks to process the last time around, wait a full second before polling again if there are
-            #  new blocks.
+            # If we had zero blocks to process the last time around, wait a full second before
+            # polling again if there are new blocks.
             if noblocks:
                 await asyncio.sleep(1)
                 if not self.monitor["running"]:
                     return
+                await self._process_instance(int(time.time()*1000), startblock, False)
+                now = int(time.time()*1000)
+                if now - last_instant >= 59500: #Roughly one minute
+                    last_instant = now
+                    await self.monitor["on_block_processed"](startblock, now)
+                if not self.monitor["running"]:
+                    return
+
             noblocks = True
-            # Get the latest ledger stats.
-            stats = await self.ledger_stats()
+            endblock, stats_error_count = await self._get_endblock()
             if not self.monitor["running"]:
                 return
-            if "status" in stats and stats["status"] == 200 and "data" in stats and "block" in stats["data"]:
-                stats_error_count = 0
-                endblock = stats["data"]["block"]
+            if endblock:
                 if endblock > startblock:
                     noblocks = False
-                    for block in range(startblock + 1, endblock + 1):
-                        grouped = dict()
-                        # Fetch the new block
-                        block_data = await self.block.query(block=block)
-                        if not self.monitor["running"]:
-                            return
-                        # Itterate all flakes.
-                        for flake in block_data[0]["flakes"]:
-                            predno = flake[1]
-                            # Patch numeric predicates to textual ones.
-                            if predno in predicate:
-                                flake[1] = predicate[predno]
-                            else:
-                                raise RuntimeError("Need a restart after new predicates are added to the database")
-                            # Group the flakes together by object.
-                            if not flake[0] in grouped:
-                                grouped[flake[0]] = list()
-                            grouped[flake[0]].append(flake)
+                    for block in range(startblock+1, endblock + 1):
+                        grouped, obj_tx, instant, block_meta = await self._get_and_preprocess_block(block)
                         # Process per object.
                         for obj in grouped:
-                            # Ectract the collection name
-                            collection = grouped[obj][0][1].split("/")[0]
-                            # Trigger on collection if in map
-                            if collection in self.monitor["listeners"]:
-                                latest = await self.flureeql.query(select=["*"], ffrom=obj)
-                                if not self.monitor["running"]:
-                                    return
-                                if latest:
-                                    latest = latest[0]
-                                else:
-                                    latest = None
-                                previous = await self.flureeql.query(select=["*"], ffrom=obj, block=block-1)
-                                if not self.monitor["running"]:
-                                    return
-                                if previous:
-                                    previous = previous[0]
-                                else:
-                                    previous = None
-                                if latest is None:
-                                    if "D" in self.monitor["listeners"][collection]:
-                                        for callback in self.monitor["listeners"][collection]["D"]:
-                                            await callback(obj, grouped[obj])
-                                            if not self.monitor["running"]:
-                                                return
-                                elif previous is None:
-                                    if "C" in self.monitor["listeners"][collection]:
-                                        for callback in self.monitor["listeners"][collection]["C"]:
-                                            await callback(obj, grouped[obj])
-                                            if not self.monitor["running"]:
-                                                return
-                                elif "U" in self.monitor["listeners"][collection]:
-                                    for updateinfo in self.monitor["listeners"][collection]["U"]:
-                                        await updateinfo["callback"](obj, grouped[obj])
-                                        if not self.monitor["running"]:
-                                            return
+                            if obj > 0:
+                                collection = self._get_flakeset_collection(grouped[obj])
+                                if collection in self.monitor["listeners"]:
+                                    await self._process_flakeset(collection, grouped[obj], obj_tx, block, block_meta)
+                                    if not self.monitor["running"]:
+                                        return
                         # Call the persistence layer.
-                        await self.monitor["on_block_processed"](block)
+                        await self.monitor["on_block_processed"](block, instant)
+                        last_instant = instant
                     # Set the new start block.
                     startblock = block
             else:
@@ -1302,7 +1714,7 @@ class _FlureeDbClient:
 
         class CommandEndpoint:
             """Endpoint for FlureeQL command"""
-            def __init__(self, api_endpoint, client):
+            def __init__(self, api_endpoint, client, ssl_verify_disabled=False):
                 """Constructor
 
                 Parameters
@@ -1311,9 +1723,11 @@ class _FlureeDbClient:
                                Name of the API endpoint
                 client: object
                         The wrapping _FlureeDbClient
+                ssl_verify_disabled: bool
+                        When using https, don't validata ssl certs.
                 """
                 self.client = client
-                self.stringendpoint = _StringEndpoint(api_endpoint, client)
+                self.stringendpoint = _StringEndpoint(api_endpoint, client, ssl_verify_disabled)
 
             async def transaction(self, transaction_obj, deps=None, do_await=True):
                 """Transact with list of python dicts that should get serialized to JSON,
@@ -1350,15 +1764,17 @@ class _FlureeDbClient:
 
         class LedgerStatsEndpoint:
             """Endpoint for ledger_stats"""
-            def __init__(self, client):
+            def __init__(self, client, ssl_verify_disabled=False):
                 """Constructor
 
                 Parameters
                 ----------
                 client: object
                         The wrapping _FlureeDbClient
+                ssl_verify_disabled: bool
+                        When using https, don't validata ssl certs.
                 """
-                self.stringendpoint = _StringEndpoint('ledger_stats', client)
+                self.stringendpoint = _StringEndpoint('ledger_stats', client, ssl_verify_disabled)
 
             async def __call__(self):
                 """Send request to ledger-stats endpoint and retrieve result
@@ -1378,7 +1794,7 @@ class _FlureeDbClient:
         if api_endpoint in ["command"]:
             if self.signer is None:
                 raise FlureeKeyRequired("Command endpoint not supported in open-API mode. privkey required!")
-            return CommandEndpoint(api_endpoint, self)
+            return CommandEndpoint(api_endpoint, self, self.ssl_verify_disabled)
         if api_endpoint == 'ledger_stats':
-            return LedgerStatsEndpoint(self)
+            return LedgerStatsEndpoint(self, self.ssl_verify_disabled)
         return FlureeQlEndpoint(api_endpoint, self, self.ssl_verify_disabled)
